@@ -4,6 +4,8 @@ L2phylo = function(L,dropextinct = T)
    age = L[1,1]
    L[,1] = age - L[,1]
    L[1,1] = -1
+   notmin1 = which(L[,4] != -1)
+   L[notmin1,4] = age - L[notmin1,4]
    if(dropextinct == T)
    {
       sall = which(L[,4] == -1)
