@@ -19,8 +19,14 @@ dd_SR_ML = function(brts, initparsopt = c(0.5,0.1,2*(1+length(brts)+missnumspec)
 # - ddmodel = diversity-dependent model,mode of diversity-dependence
 #  . ddmodel == 1 : linear dependence in speciation rate
 #  . ddmodel == 2 : exponential dependence in speciation rate
+#  . ddmodel == 1 : linear dependence in speciation rate
+#  . ddmodel == 2 : exponential dependence in speciation rate
+#  . ddmodel == 2.1: variant with offset at infinity
+#  . ddmodel == 2.2: 1/n dependence in speciation rate
 #  . ddmodel == 3 : linear dependence in extinction rate
 #  . ddmodel == 4 : exponential dependence in extinction rate
+#  . ddmodel == 4.1: variant with offset at infinity
+#  . ddmodel == 4.2: 1/n dependence in speciation rate
 # - missnumspec = number of missing species    
 # - cond = conditioning
 #  . cond == 0 : no conditioning
@@ -53,7 +59,7 @@ if(length(namepars[idparsopt]) == 0) { optstr = "nothing" } else { optstr = name
 cat("You are optimizing",optstr,"\n")
 if(length(namepars[idparsfix]) == 0) { fixstr = "nothing" } else { fixstr = namepars[idparsfix] }
 cat("You are fixing",fixstr,"\n")
-if(length(namepars[idparsnoshift]) == 0) { noshiftstr = "anything" } else { noshiftstr = namepars[idparsnoshift] }
+if(length(namepars[idparsnoshift]) == 0) { noshiftstr = "none" } else { noshiftstr = namepars[idparsnoshift] }
 cat("You are not shifting",noshiftstr,"\n")
 cat("Optimizing the likelihood - this may take a while.","\n")
 flush.console()
